// gl-utils.cc

#include <iostream>
#include <string>
#include <sstream>
#include <cstdlib>
#include <cstdio>

#include "gl-utils.h"

/*****************************************************************************
 * print_opengl_error                                                        *
 *****************************************************************************/
void print_opengl_error(const char *file, int line)
{
  GLenum error;

  while ((error = glGetError()) != GL_NO_ERROR)
    {
      std::cerr << "glError in file " << file << ", line " << line << ": " <<
        gluErrorString(error) << std::endl;
      error = glGetError();
    }
}

/*****************************************************************************
 * print_shader_info_log                                                     *
 *****************************************************************************/
static void print_shader_info_log(GLuint shader)
{
  int     info_log_length = 0;
  int     chars_written  = 0;
  GLchar *info_log;
  GLint   status;

  glGetShaderiv(shader, GL_COMPILE_STATUS, &status);           PRINT_OPENGL_ERROR();
  glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &info_log_length); PRINT_OPENGL_ERROR();

  if (info_log_length > 1)
    {
      info_log = new GLchar[info_log_length];
      glGetShaderInfoLog(shader, info_log_length, &chars_written, info_log); PRINT_OPENGL_ERROR ();
      std::cerr << "Shader InfoLog:" << std::endl << info_log << std::endl;
      delete[] info_log;
    }
  if (!status) exit(-1);
}

/*****************************************************************************
 * print_program_info_log                                                    *
 *****************************************************************************/
static void print_program_info_log(GLuint program)
{
  int     info_log_length = 0;
  int     chars_written  = 0;
  GLchar *info_log;
  GLint   status;

  glGetProgramiv(program, GL_LINK_STATUS, &status);              PRINT_OPENGL_ERROR();
  glGetProgramiv(program, GL_INFO_LOG_LENGTH, &info_log_length); PRINT_OPENGL_ERROR();

  if (info_log_length > 1)
    {
      info_log = new GLchar[info_log_length];
      glGetProgramInfoLog(program, info_log_length, &chars_written, info_log); PRINT_OPENGL_ERROR ();
      std::cout << "Program InfoLog:" << std::endl << info_log << std::endl;
      delete[] info_log;
    }
  if (!status) exit(-1);
}

/*****************************************************************************
 * get_file_content                                                          *
 *****************************************************************************/
static std::string get_file_content(const std::string &filename)
{
  std::string ret;
  if (FILE *fp = fopen(filename.c_str (), "r"))
    {
      char buf[1024];
      while (size_t len = fread(buf, 1, sizeof(buf), fp))
          ret += std::string(buf, buf + len);
      fclose(fp);
    }
  return ret;
}

/*****************************************************************************
 * read_shader                                                               *
 *****************************************************************************/
GLuint read_shader(const std::string &vertex, const std::string &fragment)
{
  std::string  source;
  const char  *cstring;
  GLuint       program;
  GLuint       handle;

  program = glCreateProgram();                 PRINT_OPENGL_ERROR();

  // Create the vertex shader.
  handle = glCreateShader(GL_VERTEX_SHADER);   PRINT_OPENGL_ERROR();
  source = get_file_content(vertex);
  cstring = source.c_str();
  glShaderSource(handle, 1, &cstring, 0);      PRINT_OPENGL_ERROR();
  glCompileShader(handle);                     PRINT_OPENGL_ERROR();
  print_shader_info_log(handle);
  glAttachShader(program, handle);             PRINT_OPENGL_ERROR();
  glDeleteShader(handle);                      PRINT_OPENGL_ERROR();

  // Create the fragment shader
  handle = glCreateShader(GL_FRAGMENT_SHADER); PRINT_OPENGL_ERROR();
  source = get_file_content(fragment);
  cstring = source.c_str();
  glShaderSource(handle, 1, &cstring, 0);      PRINT_OPENGL_ERROR();
  glCompileShader(handle);                     PRINT_OPENGL_ERROR();
  print_shader_info_log (handle);
  glAttachShader(program, handle);             PRINT_OPENGL_ERROR();
  glDeleteShader(handle);                      PRINT_OPENGL_ERROR();

  /* Link the program object and print out the info log */
  glLinkProgram(program);                      PRINT_OPENGL_ERROR();
  print_program_info_log(program);
  glUseProgram(program);                       PRINT_OPENGL_ERROR();

  return program;
}

